@@ -8,19 +8,16 @@ use std::{
 };
 
 use crate::{
-    buffer_unordered_poller_macro,
-    receiver::{Action, Event, ReciveTypedReceiver, SendUntypedReceiver},
-    receivers::{fix_type, Request},
-};
-use anyhow::Result;
-use futures::{stream::FuturesUnordered, Future, StreamExt};
-
-use super::{BufferUnorderedConfig, BufferUnorderedStats};
-use crate::{
+    buffer_unordered_poller_macro, 
     builder::ReceiverSubscriberBuilder,
-    receiver::{SendError, SendTypedReceiver},
+    error::{Error, StdSyncSendError, SendError}, 
+    receiver::{Action, Event, ReciveTypedReceiver, SendUntypedReceiver, SendTypedReceiver}, 
+    receivers::{fix_type, Request},
     Bus, Handler, Message, Untyped,
 };
+use super::{BufferUnorderedConfig, BufferUnorderedStats};
+
+use futures::{stream::FuturesUnordered, Future, StreamExt};
 use parking_lot::Mutex;
 use tokio::sync::mpsc;
 
@@ -39,11 +36,11 @@ buffer_unordered_poller_macro!(
     }
 );
 
-pub struct BufferUnorderedSync<M, R = (), E = crate::error::Error>
+pub struct BufferUnorderedSync<M, R, E>
 where
     M: Message,
     R: Message,
-    E: crate::Error,
+    E: StdSyncSendError,
 {
     tx: mpsc::UnboundedSender<Request<M>>,
     stats: Arc<BufferUnorderedStats>,
@@ -55,7 +52,7 @@ where
     T: Handler<M, Response = R, Error = E> + 'static,
     R: Message,
     M: Message,
-    E: crate::Error,
+    E: StdSyncSendError,
 {
     type Config = BufferUnorderedConfig;
 
@@ -106,7 +103,7 @@ impl<M, R, E> SendUntypedReceiver for BufferUnorderedSync<M, R, E>
 where
     M: Message,
     R: Message,
-    E: crate::Error,
+    E: StdSyncSendError,
 {
     fn send(&self, msg: Action) -> Result<(), SendError<Action>> {
         match self.tx.send(Request::Action(msg)) {
@@ -121,7 +118,7 @@ impl<M, R, E> SendTypedReceiver<M> for BufferUnorderedSync<M, R, E>
 where
     M: Message,
     R: Message,
-    E: crate::Error,
+    E: StdSyncSendError,
 {
     fn send(&self, mid: u64, m: M) -> Result<(), SendError<M>> {
         match self.tx.send(Request::Request(mid, m)) {
@@ -140,7 +137,7 @@ impl<M, R, E> ReciveTypedReceiver<R, E> for BufferUnorderedSync<M, R, E>
 where
     M: Message,
     R: Message,
-    E: crate::Error,
+    E: StdSyncSendError,
 {
     fn poll_events(&self, ctx: &mut Context<'_>) -> Poll<Event<R, E>> {
         let poll = self.srx.lock().poll_recv(ctx);
